@@ -49,18 +49,20 @@ make_exercise_transform_html <- function(converter = c("ttm", "tth", "tex2image"
         }
       }
       for(i in c("question", "questionlist", "solution", "solutionlist")) {
-        if(grepl("list", i)) {
-          j <- grep(i, inames)
-        } else {
-          j <- grep(i, inames)
-          j <- j[!grepl("list", inames[j])]
+        if(!is.null(x[[i]])) {
+          if(grepl("list", i)) {
+            j <- grep(i, inames)
+          } else {
+            j <- grep(i, inames)
+            j <- j[!grepl("list", inames[j])]
+          }
+          x[[i]] <- if(b64) {
+            dir[j]
+          } else {
+            paste("<img src=\"", file.path(sdir, basename(dir[j])), "\" alt=\"", inames[j], "\" />", sep = "")
+          }
+          names(x[[i]]) <- inames[j]
         }
-        x[[i]] <- if(b64) {
-          dir[j]
-        } else {
-          paste("<img src=\"", file.path(sdir, basename(dir[j])), "\" alt=\"", inames[j], "\" />", sep = "")
-        }
-        names(x[[i]]) <- inames[j]
       }
       if(!b64) {
         for(i in dir) {
@@ -150,6 +152,10 @@ make_exercise_transform_html <- function(converter = c("ttm", "tth", "tex2image"
       x$solution <- trex$solution
       x$solutionlist <- sapply(trex[grep("solutionlist", namtrex)], paste, collapse = "\n")
 
+      for(j in c("question", "questionlist", "solution", "solutionlist")) {
+        if(length(x[[j]]) < 1L) x[[j]] <- NULL
+      }
+
       setwd(owd)
 
       x
